@@ -10,8 +10,11 @@ import logging
 import operator
 import os
 import pickle
+import sys
 from typing import Any, Callable, Generator, Iterable
 
+import natsort
+
 import dxtbx
 from dxtbx.format.Format import Format
 from dxtbx.format.FormatMultiImage import FormatMultiImage
@@ -48,6 +51,16 @@ __all__ = [
 
 logger = logging.getLogger(__name__)
 
+# REMOVE and inline when Python 3.10 is minimum
+if sys.version_info < (3, 10):
+    scaling_model_entry_points = importlib.metadata.entry_points().get(
+        "dxtbx.scaling_model_ext", []
+    )
+else:
+    scaling_model_entry_points = importlib.metadata.entry_points(
+        group="dxtbx.scaling_model_ext"
+    )
+
 
 class InvalidExperimentListError(RuntimeError):
     """
@@ -59,14 +72,6 @@ class InvalidExperimentListError(RuntimeError):
     """
 
 
-try:
-    scaling_model_entry_points = importlib.metadata.entry_points()[
-        "dxtbx.scaling_model_ext"
-    ]
-except KeyError:
-    scaling_model_entry_points = []
-
-
 class FormatChecker:
     """A helper class to speed up identifying the correct image format by first
     trying the last format that was used."""
@@ -426,7 +431,7 @@ class ExperimentListDict:
                     # make a new bigger scan
                     o = eobj_scan[imageset_ref].get_oscillation()
                     s = scan.get_oscillation()
-                    assert o[1] == s[1]
+                    assert abs(o[1] - (s[1])) < 1e-7
                     scan = copy.deepcopy(scan)
                     scan.set_image_range((min(i[0], j[0]), max(i[1], j[1])))
                     scan.set_oscillation((min(o[0], s[0]), o[1]))
@@ -691,9 +696,34 @@ class ExperimentListFactory:
         # - Any consecutive still frames that share any metadata with the
         #   previous still fram get collected into one ImageSet
 
+        all_tof = False
+        for format_class, records in format_groups.items():
+            for i in records:
+                try:  # records can be ImageMetadataRecord or ImageSequence
+                    scan = i.get_scan()
+                    if scan is not None and scan.has_property("time_of_flight"):
+                        all_tof = True
+                    elif all_tof:
+                        raise RuntimeError(
+                            "Cannot process mix of ToF and non ToF experiments"
+                        )
+                except AttributeError:
+                    if all_tof:
+                        raise RuntimeError(
+                            "Cannot process mix of ToF and non ToF experiments"
+                        )
+
         # Treat each format as a separate block of data
         for format_class, records in format_groups.items():
             if issubclass(format_class, FormatMultiImage):
+                if all_tof:
+                    _merge_sequence_model_metadata(
+                        records,
+                        compare_beam=compare_beam,
+                        compare_detector=compare_detector,
+                        compare_goniometer=compare_goniometer,
+                    )
+
                 for imageset in records:
                     experiments.extend(
                         ExperimentListFactory.from_imageset_and_crystal(
@@ -913,8 +943,6 @@ class ExperimentListFactory:
                     f"Image file {filenames[0]} appears to be a '{type(format_class).__name__}', but this is an abstract Format"
                 )
             else:
-                index = slice(*template_string_number_index(template))
-
                 image_range = kwargs.get("image_range")
                 if image_range:
                     first, last = image_range
@@ -924,7 +952,13 @@ class ExperimentListFactory:
                 if not kwargs.get("allow_incomplete_sequences", False):
                     if "#" in template:
                         # Check all images in range are present - if allowed
-                        all_numbers = {int(f[index]) for f in filenames}
+                        i0, i1 = template_string_number_index(template)
+                        prefix = template[:i0]
+                        suffix = template[i1:]
+                        all_numbers = {
+                            int(f.replace(prefix, "").replace(suffix, ""))
+                            for f in filenames
+                        }
                         missing = set(range(first, last + 1)) - all_numbers
                         if missing:
                             raise ValueError(
@@ -1184,7 +1218,7 @@ def _openingpathiterator(pathnames: Iterable[str]):
     """
 
     # Store a tuple of (recurse, pathname) to track what was root level
-    paths = collections.deque((True, x) for x in sorted(pathnames))
+    paths = collections.deque((True, x) for x in natsort.natsorted(pathnames))
 
     while paths:
         # Get the next path from the queue
@@ -1243,6 +1277,42 @@ def _merge_model_metadata(
         )
 
 
+def _merge_sequence_model_metadata(
+    records: Iterable[ImageSequence],
+    compare_beam: Callable | None = None,
+    compare_detector: Callable | None = None,
+    compare_goniometer: Callable | None = None,
+):
+    record_altered = False
+    for prev, record in _iterate_with_previous(records):
+        if prev is None:
+            continue
+
+        record_altered = False
+        record_beam = record.get_beam()
+        record_detector = record.get_detector()
+        record_goniometer = record.get_goniometer()
+        prev_beam = prev.get_beam()
+        prev_detector = prev.get_detector()
+        prev_goniometer = prev.get_goniometer()
+
+        if record_beam is not prev_beam and compare_beam(record_beam, prev_beam):
+            record.set_beam(prev_beam)
+            record_altered = True
+        if record_detector is not prev_detector and compare_detector(
+            record_detector, prev_detector
+        ):
+            record.set_detector(prev_detector)
+            record_altered = True
+        if record_goniometer is not prev_goniometer and compare_goniometer(
+            record_goniometer, prev_goniometer
+        ):
+            record.set_goniometer(prev_goniometer)
+            record_altered = True
+
+    return record_altered
+
+
 def _merge_scans(
     records: Iterable[ImageMetadataRecord], scan_tolerance: float | None = None
 ) -> list[ImageMetadataRecord]:
